from glpk import *

def problem0():
    print "\n>>problem 0:\n"    
    problem = glpk("models/samp1.mps")
    problem.solve()
    
    print problem.solution()
    problem = glpk("models/plan.lp")
    problem.solve()
    print problem.solution()

def problem1():
    print "\n>>problem 1:\n"
    problem = glpk("models/net3.mod","models/net3.dat")
    problem.solve()
    sol1 = problem.solution()
    
    problem.pd_cap['NE'] = 10000
    problem.dw_cap['NE', 'BOS'] = 50
    problem.p_supply = 450
    problem.update()
    problem.solve()
    sol2 = problem.solution()
    
    print "solution1:", sol1
    print "solution2:", sol2

def problem2():
    print "\n>>problem 2:\n"
    problem = glpk("models/multi.mod","models/multi.dat")
    problem.update()
    problem.solve()
    print problem.supply['GARY','bands']
    print "solution:", problem.solution()

def problem3():
    print "\n>>problem 3:\n"
    problem = glpk("models/steel.mod","models/steel.dat")
    problem.update()
    problem.solve()
    print "solution:", problem.solution()
    print "problem.total_profit =", problem.total_profit
    print "problem.Make =", problem.Make
    
def problem4():
    print "\n>>problem 4:\n"
    problem = glpk("models/steel.mod")
    problem.PROD = ['bands', 'coils']
    problem.rate['bands'] = 200
    problem.rate['coils'] = 140
    problem.profit['bands'] = 25
    problem.profit['coils'] = 30
    problem.market['bands'] = 6000
    problem.market['coils'] = 4000
    problem.avail = 40
    problem.update()
    problem.solve()
    print "problem.solution() =", problem.solution()
    print "problem.sets() =", problem.sets()
    print "problem.parameters() =", problem.parameters()
    print "problem.variables() =", problem.variables()
    print "problem.constraints() =", problem.constraints()
    print "problem.objectives() =", problem.objectives()
    print "problem.Make =", problem.Make
    print "problem.Make['bands'] =", problem.Make['bands']
    print "problem.Make['bands'].value() =", problem.Make['bands'].value()    
    print "problem.total_profit =", problem.total_profit
    print "problem.total_profit.value() =", problem.total_profit.value()
    problem.Make >= 10
    problem.Make['coils'] <= 100
    0 <= problem.total_profit <= 100000

    problem.Make <= 15
    problem.Make['bands'] <= 10
    print problem.Make['bands']._bounds()
    #problem.Time <= 10
    
    
    print "problem.total_profit =", problem.total_profit    
    problem.solve()
    #print "problem.Make.bounds() =", problem.Make._bounds()
    #print "problem.Make['coils'].bounds() =", problem.Make['coils']._bounds()
    #print "problem.Make['bands'].bounds() =", problem.Make['bands']._bounds()       
    print "problem.solution() =", problem.solution()

if __name__ == "__main__":
    problem0()
    problem1()
    problem2()
    problem3()
    problem4()




from glpk import *

# create an LP as an instance of class "glpk",
# based on the model "diet.mod" (coming in glpk's "examples")
problem = glpk("models/diet.mod")

# solve
problem.solve()
print "\n\nsolution:", problem.solution()

# define/modify problem data:
# "F" and "a" are, respectively, a "set" and a "param" in "diet.mod";
# now, they can be accessed in Python as members of the "glpk" instance
problem.F += ['Bacalhau']	# add a new food
problem.a['Bacalhau','Calorie']    = 87.3	# nutrients
problem.a['Bacalhau','Protein']	   = 98.7
problem.a['Bacalhau','Calcium']	   = 17.3
problem.a['Bacalhau','Iron']	   = 12.3
problem.a['Bacalhau','Vitamin-A']  = 12.3	

# update and solve
problem.update()
problem.solve()
print "\n\nsolution:", problem.solution()

# one more update
problem.F.remove('Bacalhau')	# remove food
for key in problem.a.keys():
    if 'Bacalhau' in key:
        del problem.a[key]

# update and solve
problem.update()
problem.solve()
print "\n\nsolution:", problem.solution()



from glpk.glpkpi import *
size = 1000+1
ia = intArray(size)
ja = intArray(size)
ar = doubleArray(size)
prob = glp_create_prob()

glp_set_prob_name(prob, "sample")
glp_set_obj_dir(prob, GLP_MAX)
glp_add_rows(prob, 3)
glp_set_row_name(prob, 1, "p")
glp_set_row_bnds(prob, 1, GLP_UP, 0.0, 100.0)
glp_set_row_name(prob, 2, "q")
glp_set_row_bnds(prob, 2, GLP_UP, 0.0, 600.0)
glp_set_row_name(prob, 3, "r")
glp_set_row_bnds(prob, 3, GLP_UP, 0.0, 300.0)
glp_add_cols(prob, 3)
glp_set_col_name(prob, 1, "x1")
glp_set_col_bnds(prob, 1, GLP_LO, 0.0, 0.0)
glp_set_obj_coef(prob, 1, 10.0)
glp_set_col_name(prob, 2, "x2")
glp_set_col_bnds(prob, 2, GLP_LO, 0.0, 0.0)
glp_set_obj_coef(prob, 2, 6.0)
glp_set_col_name(prob, 3, "x3")
glp_set_col_bnds(prob, 3, GLP_LO, 0.0, 0.0)
glp_set_obj_coef(prob, 3, 4.0)
ia[1] = 1; ja[1] = 1; ar[1] =  1.0	# /* a[1,1] =  1 */
ia[2] = 1; ja[2] = 2; ar[2] =  1.0	# /* a[1,2] =  1 */
ia[3] = 1; ja[3] = 3; ar[3] =  1.0	# /* a[1,3] =  1 */
ia[4] = 2; ja[4] = 1; ar[4] = 10.0	# /* a[2,1] = 10 */
ia[5] = 3; ja[5] = 1; ar[5] =  2.0	# /* a[3,1] =  2 */
ia[6] = 2; ja[6] = 2; ar[6] =  4.0	# /* a[2,2] =  4 */
ia[7] = 3; ja[7] = 2; ar[7] =  2.0	# /* a[3,2] =  2 */
ia[8] = 2; ja[8] = 3; ar[8] =  5.0	# /* a[2,3] =  5 */
ia[9] = 3; ja[9] = 3; ar[9] =  6.0	# /* a[3,3] =  6 */

glp_load_matrix(prob, 9, ia, ja, ar)
glp_simplex(prob, None)
Z = glp_get_obj_val(prob)
x1 = glp_get_col_prim(prob, 1)
x2 = glp_get_col_prim(prob, 2)
x3 = glp_get_col_prim(prob, 3)
print "\nZ = %g; x1 = %g; x2 = %g; x3 = %g\n" %(Z, x1, x2, x3)
del prob



"""
   Filename:  glpk_lex.py (Python lexer for GnuMathProg language)

-- This code is part of the Python-GLPK interface.
--
-- Copyright (C) 2005, Joao Pedro Pedroso and Filipe Brandao
-- Faculdade de Ciencias, Universidade do Porto
-- Porto, Portugal. All rights reserved. E-mail: <jpp@fc.up.pt>.
--
-- Python-GLPK is free software; you can redistribute it and/or modify it
-- under the terms of the GNU General Public License as published by the
-- Free Software Foundation; either version 2, or (at your option) any
-- later version.
--  
-- Python-GLPK is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-- General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with GLPK; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-- 02110-1301, USA.
"""

import ply.lex as lex
import re

reserved = {}

# List of token names.   This is always required
tokens = [
   'LPAREN',
   'RPAREN',
   'LBRACK',
   'RBRACK',
   'STRING',
   'INTEGER',
   'FLOAT',
   'PROMPT',
] + list(reserved.values())

# Regular expression rules for simple tokens
t_LPAREN = r'\('
t_RPAREN = r'\)'
t_LBRACK = r'\['
t_RBRACK = r'\]'
t_PROMPT = r'>>>'

literals = ":;=,"

def t_STRING(t):
    r'''[a-zA-Z_0-9]*[a-zA-Z_+][a-zA-Z_\+\-0-9]*
        | [a-zA-Z_][a-zA-Z_\+\-0-9]* 
        | [a-zA-Z_\+\-0-9]*[a-zA-Z_]
        | \'([^\']|\'\')*\' | \"([^\"]|\"\")*\" '''
    t.value = t.value.strip('\'\"\n')
    t.value = t.value.replace('\'','')
    t.value = t.value.replace('\"','')
    return t

def t_FLOAT(t):
    r'''[\+\-]?[0-9]*[\.]?[0-9]*([eE][\-\+]?[0-9]+) 
        | [\+\-]?[0-9]*\.[0-9]+'''
    t.value = float(t.value)
    return t
    
def t_INTEGER(t):
    r'[\+\-]?\d+'
    t.value = int(t.value)    
    return t



# Define a rule so we can track line numbers
def t_NEWLINE(t):
    r'\n+'
    t.lexer.lineno += len(t.value)

# A string containing ignored characters
t_ignore  = ' \r\t,'

# Error handling rule
def t_error(t):
    print "Illegal character %s" % repr(t.value[0])
    t.lexer.skip(1)

# Build the lexer
lexer = lex.lex()
#lexer = lex.lex(optimize=1)





"""
   Filename:  glpk.py (define a class for calling GLPK from Python)

-- This code is part of the Python-GLPK interface.
--
-- Copyright (C) 2005, Joao Pedro Pedroso and Filipe Brandao
-- Faculdade de Ciencias, Universidade do Porto
-- Porto, Portugal. All rights reserved. E-mail: <jpp@fc.up.pt>.
--
-- Python-GLPK is free software; you can redistribute it and/or modify it
-- under the terms of the GNU General Public License as published by the
-- Free Software Foundation; either version 2, or (at your option) any
-- later version.
--  
-- Python-GLPK is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-- General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with GLPK; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-- 02110-1301, USA.
"""
from model_objects import *
from glpk_parser import *
from glpkpi import *
from array import *
import sys
import os
import tempfile
LOG = False
VERBOSE = False
DEBUG = False

GLPK_VERSION = glp_version();

class glpk:
    def __init__(self, mod_file = None, dat_file = None):
        ### # parameter structure:    NOT USED YET
        self._parm = glp_smcp();
        glp_init_smcp(self._parm);
        #self._parm.msg_lev = GLP_MSG_OFF;    # silent modes
        #self._parm.msg_lev = GLP_MSG_ALL;
        self._parm.msg_lev = GLP_MSG_ERR;
        #self._parm.meth = GLP_DUAL;
        self._parm.meth = GLP_DUALP;
        #self._parm.meth = GLP_PRIMAL;
        
        self._tran = None
        self._lp = None
        self._type = {}
        self._model = ""
        self._ptype = None
        self._bounds = {}
        self._ready = False

        if mod_file == None:
            if VERBOSE:
                print "creating problem -- no file associated"
            self._lp = glp_create_prob();
        elif mod_file.find(".mps") != -1:
            if VERBOSE:
                print "creating problem from mps file"
            self._ptype = "mps"
            self._lp = glp_create_prob();
            ret = glp_read_mps(self._lp, GLP_MPS_FILE, None, mod_file);# !self._parm
            if ret != 0:
                print "Error reading model", mod_file
                raise IOError
            self._read_variables()
        elif mod_file.find(".lp") != -1:
            if VERBOSE:
                print "creating problem from lp file"
            self._ptype = "lp"
            self._lp = glp_create_prob();
            ret = glp_read_lp(self._lp, None, mod_file);# !self._parm
            self._read_variables()
        elif mod_file.find(".mod") != -1:
            if VERBOSE:
                print "creating problem from mod file"
            self._ptype = "mod"
            t = mod_file.rfind('.')
            if t<0: name = mod_file
            else: name = mod_file[:t]
            f, tmp_file = tempfile.mkstemp()
            
            # genereate a temporary model file
            parser = glpk_parser(mod_file, dat_file)
            parser.generate_model(tmp_file)
            if VERBOSE:
                print "temporary model file '%s' created" % (tmp_file)
            
            # read model           
            mod_file = tmp_file
            self._lp = glp_create_prob();
            self._tran = glp_mpl_alloc_wksp();
            ret = glp_mpl_read_model(self._tran, mod_file, 0);
            if ret != 0:
                print "Error reading model", mod_file
                raise IOError
            
            # load data        
            #glp_term_out(GLP_OFF);    
            f, output = tempfile.mkstemp()
            os.close(f)
            ret = glp_mpl_generate(self._tran, output)
            if ret != 0:
                print "Error generating model"
                raise IOError
            load_data(parser, output, self)
            if not DEBUG:
                os.remove(output)
                os.remove(tmp_file)
            #glp_term_out(GLP_ON);
            self._cols = self._rows = None
     
    def _delete(self):
        if self._tran and glp_mpl_free_wksp:
            glp_mpl_free_wksp(self._tran);
        if self._lp and glp_delete_prob:
            glp_delete_prob(self._lp);
      
    def _reload(self):
        if self._ptype != 'mod': return
        self._delete()
        # read model
        s = self._dump()
        f, mod_file = tempfile.mkstemp()
        os.write(f, s)
        os.close(f)
        self._lp = glp_create_prob();
        self._tran = glp_mpl_alloc_wksp();        
        ret = glp_mpl_read_model(self._tran, mod_file, 0);
        if ret != 0:
            print "Error reading model", mod_file
            raise IOError        
        #if not VERBOSE: glp_term_out(GLP_OFF);            
        ret = glp_mpl_generate(self._tran, None);
        if ret != 0:
            print "Error generating model"
            raise IOError
        glp_mpl_build_prob(self._tran, self._lp);
        #if not VERBOSE: glp_term_out(GLP_ON);
        if not DEBUG:
            os.remove(mod_file)
        
    def _dump(self):
        if self._ptype != 'mod': return
        s = self._model+"\ndata;\n"       
        for name in self._sets_names:
            if type(self[name])!=dict:
                val = str(self[name]).strip('[]')
                s += "set %s := %s;\n" % (name,val)    
            else:
                for subname in sorted(self[name]):
                    if type(subname)==str: 
                        sname = '\''+subname+'\''
                    elif type(subname)!=tuple:
                        sname = str(subname)
                    else:
                        sname = str(subname).strip('()')
                    val = str(self[name][subname]).strip('[]')
                    s += "set %s[%s] := %s;\n" % (name,sname,val)
        for name in self._params_names:
            if type(self[name])!=dict:
                if name not in self._default or self[name] != None:
                    s += "param %s := %s;\n" % (name,self[name])
                else:
                    val = self[name]
                    if val==None: val = ""
                    s += "param %s default %s := %s;\n" % (name,self._default[name],val)
            else:
                if name not in self._default:
                    s += "param %s :=" % (name)
                else:
                    s += "param %s default %s :=" % (name,self._default[name])
                for subname in sorted(self[name]):
                    if type(subname)==str: 
                        sname = '\''+subname+'\''
                    elif type(subname)!=tuple:
                        sname = str(subname)
                    else:
                        sname = str(subname).strip('()')
                    s += " [%s] %s" % (sname,self[name][subname]) 
                s += ";\n"                
        s += "end;\n"
        return s
 
    def _get_col_bnds(self, i):
        lb = glp_get_col_lb(self._lp, i);
        ub = glp_get_col_ub(self._lp, i);
        tp = glp_get_col_type(self._lp, i);
        bnds = (None, None)            
        if tp == GLP_LO:
            bnds = (('>=',lb),None)
        elif tp == GLP_UP:
            bnds = (None,('<=',ub))
        elif tp == GLP_DB:
            bnds =(('>=',lb),('<=',ub))
        elif tp == GLP_FX:
            bnds = ('=',lb)
        return bnds
        
    def _get_row_bnds(self, i):
        lb = glp_get_row_lb(self._lp, i);
        ub = glp_get_row_ub(self._lp, i);
        tp = glp_get_row_type(self._lp, i);
        bnds = (None, None)            
        if tp == GLP_LO:
            bnds = (('>=',lb),None)
        elif tp == GLP_UP:
            bnds = (None,('<=',ub))
        elif tp == GLP_DB:
            bnds =(('>=',lb),('<=',ub))
        elif tp == GLP_FX:
            bnds = ('=',lb)
        return bnds
        
    def _get_bounds(self, name):
        rows = self._rows
        cols = self._cols
        if name in rows:
            p = rows[name]
            if len(p)!=1:
                return None    
            return self._get_row_bnds(p[0])
        elif name in cols:
            p = cols[name]
            if len(p)!=1:
                return None            
            return self._get_col_bnds(p[0])
        else:
            return None
        
    def _read_variables(self):
        bounds = self._default_bounds = {}  
        
        cols = self._cols = {} 
        for i in xrange(1,glp_get_num_cols(self._lp)+1):
            name = glp_get_col_name(self._lp, i)
            tname = name[:max(0,name.find('['))]         
            bounds[name] = self._get_col_bnds(i)
            cols[name] = [i]
            if tname != "":
                if tname not in self._cols:
                    cols[tname] = [i]
                else:
                    cols[tname].append(i)
                    
        rows = self._rows = {}
        for i in xrange(1,glp_get_num_rows(self._lp)+1):
            name = glp_get_row_name(self._lp, i)
            tname = name[:max(0,name.find('['))]
            bounds[name] = self._get_row_bnds(i)
            rows[name] = [i]
            if tname != "":
                if tname not in self._cols:
                    rows[tname] = [i]
                else:
                    rows[tname].append(i)
        
    def _apply_bounds(self):
        rows = self._rows
        cols = self._cols
        types = {
            (None,None): GLP_FR,
            ('>=',None): GLP_LO,
            (None,'<='): GLP_UP,
            ('>=','<='): GLP_DB,
            ('='): GLP_FX
        }
        
        def conv(bnds):
            if bnds[0] == '=':
                return GLP_FX, bnds[1], bnds[1]
            else:
                l, lb = bnds[0], None
                if l != None: l,lb = l                  
                u, ub = bnds[1], None
                if u != None: u, ub = u
                if lb == None: lb = 0.0
                if ub == None: ub = 0.0
                return types[l,u], lb, ub
                
        for name in sorted(self._bounds):
            bnds = self._bounds[name]
            if len(bnds)==2 and name in self._default_bounds:
                dbnds = self._default_bounds[name]
                if dbnds[0]!='=':
                    if bnds[0]==None:
                        bnds = dbnds[0], bnds[1]
                    if bnds[1]==None:
                        bnds = bnds[0], dbnds[1]
            tp, lb, ub = conv(bnds)
            if name in cols:
                for i in cols[name]:
                    glp_set_col_bnds(self._lp, i, tp, lb, ub);
            elif name in rows:
                for i in rows[name]:
                    glp_set_row_bnds(self._lp, i, tp, lb, ub);
            else:
                print 'model object not found!'
        
    def _set_bounds(self, varname,(_type,_value)):
        dic = self._bounds
        if varname in dic:
            bnds = dic[varname]
            if bnds[0] == '=':
                bnds = (None, None)
        else:
            bnds = dic[varname] = (None, None)
        if _type == '=':
            bnds = ('=', _value)
        elif _type == '<=':
            bnds = (bnds[0], (_type,_value))
        elif _type == '>=':
            bnds = ((_type,_value), bnds[1])        
        dic[varname] = bnds
        
    def _rm_bounds(self, varname):
        dic = self._bounds
        if varname in dic:
            del dic[varname]

    def _instantiate_solution(self):
        sol = {}
        name = glp_get_obj_name(self._lp)
        if glp_get_num_int(self._lp) == 0:  # problem is continuous
            value = glp_get_obj_val(self._lp)
            sol[name] = value
            for i in xrange(1,glp_get_num_cols(self._lp)+1):
                name = glp_get_col_name(self._lp, i)
                value = glp_get_col_prim(self._lp, i)
                sol[name] = value
        else:   # problem is MIP
            value = glp_mip_obj_val(self._lp)
            sol[name] = value
            for i in xrange(1,glp_get_num_cols(self._lp)+1):
                name = glp_get_col_name(self._lp, i)
                if glp_get_col_kind(self._lp, i) == GLP_CV:
                    value = glp_mip_col_val(self._lp, i)
                elif glp_get_col_kind(self._lp, i) in [GLP_IV, GLP_BV]:
                    # to avoid finite precision problems:
                    value = int(round(glp_mip_col_val(self._lp, i),0))
                else:
                    print "unkown col kind"
                    raise AttributeError
                sol[name] = value             
        self._sol = sol
        load_solution(sol, self)        
        
    def __del__(self):
        print "deleting glpk object"
        self._delete()         
        
    def __getitem__(self, name):
        return self.__dict__[name]
        
    def __setitem__(self, name, value):
        self.__dict__[name] = value
        
    def __setattr__(self, name, value):
        self.__dict__[name] = value
     
    def __iter__(self):
        for name in self.__dict__:
            if name in self._type:
                yield name

    # interface
    def update(self):
        self._dump()
        self._reload()
        self._ready = True

    def solution(self):
        try:
            return self._sol
        except:
            return None

    def sets(self):
        try:
            return self._sets_names
        except:
            return None    

    def parameters(self):
        try:
            return self._params_names
        except:
            return None  

    def variables(self):
        try:
            return self._vars_names
        except:
            return None

    def constraints(self):
        try:
           return self._constraints_names
        except:
            return None 

    def objectives(self):
        try:
           return self._objectives_names
        except:
            return None

    def typeof(self, obj):
        try:
            return self._type[obj]
        except:
            return None

    def solve(self, instantiate = True, bounds = True):
        #glp_term_out(GLP_OFF);
        if not self._ready:
            self.update()
        #glp_term_out(GLP_ON);
        if self._cols == None or self._rows == None:
            self._read_variables()
        if bounds:
            self._apply_bounds()
        if glp_get_num_int(self._lp) == 0:   # problem is continuous
            res = glp_simplex(self._lp, self._parm) # self._parm !!!
        else:   # problem is MIP
            if self._tran:
                glp_mpl_build_prob(self._tran, self._lp);
            res = glp_simplex(self._lp, self._parm);  # ??? should use dual simplex ???
            glp_intopt(self._lp, None);
            if self._tran:
                ret = glp_mpl_postsolve(self._tran, self._lp, GLP_MIP);
                if ret != 0:
                    print "Error on postsolving model"
                    raise AttributeError
        if instantiate:
            self._instantiate_solution()     
        if res != 0:
            return None
        else:
            return glp_get_obj_val(self._lp);        




from distutils.core import setup, Extension

# Directory containing libglpk
GLPK_LIB_DIR = '/usr/lib'       

# Directory containing glpk.h
GLPK_INC_DIR = '/usr/include'  

glpkpi = Extension('_glpkpi', libraries = ['glpk'],
                 include_dirs = [ GLPK_INC_DIR ],
                 library_dirs = [ GLPK_LIB_DIR ],
                 sources = ['swig/glpkpi_wrap.c', 'swig/glpkpi.c'] )
extmods = [glpkpi];

setup (name = 'glpk', 
    description = 'python-glpk package',
    version = '0.4.52', 
    long_description = """
Python-glpk is a free software package providing bindings in the
Python programming language for the GLPK linear and integer
optimization package.  Its main purpose is to make the development of
software for optimization applications straightforward by building on
Python's extensive standard library and on the strengths of Python as
a high-level programming language.""", 
    author = 'J.P. Pedroso and F. Brandao',
    author_email = 'jpp@fc.up.pt, fdabrandao@gmail.com',
    url = 'http://www.dcc.fc.up.pt/~jpp/code/python-glpk',
    license = 'GNU GPL version 3',
    ext_package = "glpk",
    ext_modules = extmods,
    package_dir = {"glpk": "python"},
    packages = ["glpk"])



"""
   Filename:  glpk_lex.py (Python parser for GnuMathProg language)

-- This code is part of the Python-GLPK interface.
--
-- Copyright (C) 2005, Joao Pedro Pedroso and Filipe Brandao
-- Faculdade de Ciencias, Universidade do Porto
-- Porto, Portugal. All rights reserved. E-mail: <jpp@fc.up.pt>.
--
-- Python-GLPK is free software; you can redistribute it and/or modify it
-- under the terms of the GNU General Public License as published by the
-- Free Software Foundation; either version 2, or (at your option) any
-- later version.
--  
-- Python-GLPK is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-- General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with GLPK; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-- 02110-1301, USA.
"""

import re
import ply.lex as lex
import ply.yacc as yacc

# Get the token map from the lexer.
from glpk_lex import *
from model_objects import *

class regexcache:
    def __init__(self):
        self.cache = {}
    def __getitem__(self, pattern):
        if pattern not in self.cache:
            rgx = re.compile(pattern,re.X)
            self.cache[pattern] = rgx
        else:
            rgx = self.cache[pattern]
        return rgx

regex = regexcache()

non_scolon = r'''(?:(?:\".*?\"|\'.*?\'|[^;])*;)'''

class glpk_parser:
    def __init__(self, mod_file, dat_file = None):
        try:
            fmod = open(mod_file)
            self.mod = fmod.read()
        except:
            self.mod = ""
        try:
            fdat = open(dat_file)
            self.dat = fdat.read()
        except:
            self.dat = None
        self.parse()
        
    def __getitem__(self, name):
        return self.__dict__[name]
        
    def __setitem__(self, name, value):
        self.__dict__[name] = value

    def parse_data_decls(self, ldat):
        defaults = {} # parameters default values

        pattern = r'(param)\s+(\w+)\s*((default)\s*(\w*))?'
        rgx_param = regex[pattern]
        
        pattern = r'(param)\s*((default)\s*([^\s]))?\s*(:[^:]*)?:([^:]*):'
        rgx_param_tab = regex[pattern]

        for decl in ldat:
            m = rgx_param_tab.match(decl)
            if m != None:
                lst = m.group(6).split()
                for name in lst:
                    defval = m.group(4)
                    if defval:
                        defaults[name] = defval
                continue

            m = rgx_param.match(decl)
            if m != None:
                name = m.group(2)
                defval = m.group(5)
                if defval:
                    defaults[name] = defval
                continue            
        self.default_values = defaults

    def parse_model_decls(self, lmod):
        types = {}  # set/param/var
        values = {} # {}/None
        names = []
        
        pattern = r'''(?:(set|param|var|subject\s*to|s.t.|minimize|maximize)\s+)?
                      (\w*)\s*(\"(?:[^\"]|\"\")*\"|\'(?:[^\']|\'\')*\')?
                      \s*(\{[^}]*\})?.*'''
        rgx = regex[pattern]
        
        for decl in lmod:
            m = rgx.match(decl)
            if m != None:
                _type = m.group(1)
                if _type == None:
                    _type = 's.t.'
                _name = m.group(2)
                _domain = m.group(4)
                names.append(_name)
                if _type == 's.t.':
                    types[_name] = 'constraint'
                if _type.startswith('subject'):
                    types[_name] = 'constraint'
                elif _type in ['maximize','minimize']:
                    types[_name] = 'objective'
                elif _type == 'set' and ':=' in decl:
                    types[_name] = 'setx'
                else:
                    types[_name] = _type
                if _domain!=None:
                    values[_name] = {}
                else:
                    values[_name] = None                                       
                    
        self.names = names
        self.types = types
        self.values = values
        
    def parse(self):
        mod = self.mod
        dat = self.dat
        lmod = []
        ldat = []
        
        # remove comments (#... and /*...*/)
        pattern = r'''(\".*?\"|\'.*?\'|/\*.*?\*/|\#.*)'''
        rgx = regex[pattern]
        def comrepl(match):
            s = match.group(0)
            if s[0] in '/#': return ''
            else: return s
        mod = rgx.sub(comrepl,mod)
        if dat != None:
            dat = rgx.sub(comrepl,dat)
        
        # parse model file
        pattern = r'[A-Za-z_]'+ non_scolon
        rgx = regex[pattern]
        tmod = rgx.findall(mod)

        # read data section
        i, n = 0, len(tmod)
        set_dacls = {}
        param_decls = {}
        while i < n:
            if tmod[i].startswith('data'):
                i += 1
                ldat = []
                while i < n and not tmod[i].startswith('end'):
                    ldat.append(tmod[i])
                    i += 1
                i += 1
            else:
                if tmod[i].replace(' ','') != 'end;':
                    lmod.append(tmod[i])
                i += 1

        # parse data file
        if dat != None:
            pattern = r'''set(?:(?:\".*?\"|\'.*?\'|[^;])*;)
                        | param(?:(?:\".*?\"|\'.*?\'|[^;])*;)'''
            rgx = regex[pattern]
            ldat = rgx.findall(dat)

        # parse model decls
        self.parse_model_decls(lmod)            
            
        # parse data decls
        self.parse_data_decls(ldat)
        
        # store model
        s = ""
        for decl in lmod:
            s += decl+'\n'
        self.model = s

        self.lmod = lmod
        self.ldat = ldat
                        
    def generate_model(self, filename):
        s = ""
        lmod = self.lmod
        ldat = self.ldat
        types = self.types
        for x in lmod:
            if x.startswith('set') or x.startswith('var') or x.startswith('param'):
                s += x+'\n'
        for x in types:
            if types[x] in ['set','setx','param']:
                s += "display %s;\n" % x
        s += "data;\n"
        for decl in ldat:
            s += decl+'\n'
        s += "end;"
        f = open(filename, "w")
        print >>f, s

def load_data(parser, outputfile, glpkobj):    
    text = open(outputfile).read()
    pattern = r'''Display\sstatement\sat\sline\s[0-9]+
                | \w+\shas\sempty\scontent
                | \w+\sis\sempty'''
    rgx = regex[pattern]
    text = rgx.sub('', text)
    rgx = regex[r'(\w+)\s*(\=|\:)']
    text = rgx.sub(r'>>> \1 \2', text)
    rgx = regex[r'(\w+)\s*(\[.*?\])\s*(\=|\:)']
    text = rgx.sub(r'>>> \1\2 \3', text)
    lst = yacc.parse(text)
    
    for x in parser.names:
        _type = parser.types[x]
        _value = parser.values[x]
        glpkobj._type[x] = _type
        if _type == 'var':    
            glpkobj[x] = var(glpkobj, x, _value)
        elif _type == 'constraint':
            glpkobj[x] = constraint(glpkobj, x, _value)
        elif _type == 'objective':
            glpkobj[x] = objective(glpkobj, x, _value)
        else:
            glpkobj[x] = _value 

    for (_name, _ind, _val) in lst:
        if _ind == None:
            glpkobj[_name] = _val
        else:
            glpkobj[_name][_ind] = _val 
            
    glpkobj._model = parser.model
    glpkobj._default = parser.default_values
    
    glpkobj._sets_names = []
    glpkobj._vars_names = []
    glpkobj._params_names = []
    glpkobj._constraints_names = []
    glpkobj._objectives_names = []
    for x in parser.names:
        typ = parser.types[x]
        if typ == 'set':
            glpkobj._sets_names.append(x)
        elif typ == 'param':
            glpkobj._params_names.append(x)
        elif typ == 'var':
            glpkobj._vars_names.append(x)
        elif typ == 'constraint':
            glpkobj._constraints_names.append(x)
        elif typ == 'objective':
            glpkobj._objectives_names.append(x)

def load_solution(solution, glpkobj):
    s = ""
    for x in solution:
        s +=  ">>> %s = %s\n" % (x,float(solution[x]))
    lst = yacc.parse(s)
    for (_name, _ind, _val) in lst:
        if _ind == None:
            try:
                glpkobj[_name]._value = _val     
            except:
                glpkobj[_name] = var(glpkobj, _name, _val)
        else:
            glpkobj[_name][_ind]._value = _val
            

def p_data(p):
    '''data : data decl
            |''' 
    if len(p) == 1:
        p[0] = []
    else:
        p[0] = p[1]
        p[0].append(p[2])
                    
def p_decl(p):
    '''decl : PROMPT set
            | PROMPT param'''
    p[0] = p[2]

def p_name(p):
    '''name : STRING
            | STRING tuple'''
    if len(p) == 2:
        p[0] = (p[1],None)
    else:
        if len(p[2])==1:
            p[2] = p[2][0]
        p[0] = (p[1],p[2])

def p_set(p):
    '''set :  name ':' list'''
    p[0] = (p[1][0],p[1][1],p[3])

def p_param(p):
    '''param : name '=' record'''
    p[0] = (p[1][0],p[1][1],p[3])

def p_list(p):
    '''list : record
            | list record'''
    if len(p)==2:
        p[0] = [p[1]]
    else:
        p[0] = p[1]
        p[0].append(p[2])

def p_tuple(p):
    '''tuple : LPAREN list RPAREN
             | LBRACK list RBRACK'''
    p[0] = tuple(p[2])
    
def p_record(p):
    '''record : FLOAT
              | INTEGER
              | STRING
              | tuple'''          
    p[0] = p[1]

def p_error(p):
    # Error rule for syntax errors
    print "[",p,"]"
    print "<",p.value,">"
    print "Syntax error in input!"
    import sys
    sys.exit(0)



def show_tokens():
    while True:
        tok = lexer.token()
        if not tok: break
        print tok

# Build the parser
yacc.yacc(write_tables=0, debug=0)




import glpk

print "starting..."
example = glpk.glpk("example.mod")
example.update()
example.solve()
print "solution:", example.solution()
print "solution is also here: x =", example.x, "y =", example.y




"""
   Filename:  model_objects.py (tools for processing GnuMathProg language)

-- This code is part of the Python-GLPK interface.
--
-- Copyright (C) 2005, Joao Pedro Pedroso and Filipe Brandao
-- Faculdade de Ciencias, Universidade do Porto
-- Porto, Portugal. All rights reserved. E-mail: <jpp@fc.up.pt>.
--
-- Python-GLPK is free software; you can redistribute it and/or modify it
-- under the terms of the GNU General Public License as published by the
-- Free Software Foundation; either version 2, or (at your option) any
-- later version.
--  
-- Python-GLPK is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-- General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with GLPK; see the file COPYING. If not, write to the Free
-- Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-- 02110-1301, USA.
"""

def name_to_index(name):
    if type(name)==tuple:
        s = ''
        for i in range(len(name)):
            if i>0: s += ','
            if type(name[i])==str:
                if ',' not in name[i]:
                    t = name[i]
                else:
                    t = "'%s'" % (name[i])
            else:
                t = str(name[i])
            s += t
        return s
    else:
        return name

def join_bounds(cur, new):
    if cur == None:
        return new
    if new == None:
        return cur
    if new[0] == '=':
        return new
    else:
        cur = list(cur)
        if cur[0] == '=':
            cur = (None, None)
        if new[0] != None:
            cur[0] = new[0]
        if new[1] != None:
            cur[1] = new[1]
        return tuple(cur)
        
class var:
    def __init__(self, prob, name, value = None, bnds = None):
        self._prob = prob
        self._name = name
        self._subvars = None
        if type(value) != dict:
            self._value = value
        else:
            self._subvars = value

    def value(self):
        return self._value                     

    def _bounds(self):
        try:
            bnds = self._prob._bounds[self._name]
        except:
            bnds = None
        return bnds

    def _clear(self):
        self._prob._rm_bounds(self._name)                  
        
    def __del__(self):
        self.clear()
                
    def _set_bounds(self, bounds):
        self._prob._set_bounds(self._name, bounds)
        if self._subvars:
          for x in self:
            self[x]._set_bounds(bounds)
        
    def __eq__(self, value):
        self._set_bounds(('=', value))
        for x in self:
            self[x] == value
        return nothing()
        
    def __ge__(self, value):
        self._set_bounds(('>=', value))
        return nothing()
        
    def __le__(self, value):
        self._set_bounds(('<=', value))
        return nothing()
               
    def __str__(self):
        if self._subvars == None:
            return '(var:'+str(self._value)+')'
        else:
            return '(var:'+str(self._subvars)+')'

    def __repr__(self):
        return str(self)      
            
    def __getitem__(self, name):
        if self._subvars == None: return
        name = name_to_index(name)
        itemname = "%s[%s]" % (self._name, name)
        if name not in self._subvars:
            _var = var(self._prob, itemname) 
            self._subvars[name] = _var
        else:
            _var = self._subvars[name]
        return _var
        
    def __setitem__(self, name, value):
        if self._subvars == None: return
        name = name_to_index(name)
        itemname = "%s[%s]" % (self._name, name)
        new_var = var(self._prob, itemname) 
        self._subvars[name] = new_var
        new_var == value
        
    def __iter__(self):
        if self._subvars == None: return
        for x in self._subvars:
            yield x

class constraint(var):
    def __str__(self):
        if self._subvars == None:
            return '(constraint:'+str(self._value)+')'
        else:
            return '(constraint:'+str(self._subvars)+')'

class objective(var):
    def __str__(self):
        if self._subvars == None:
            return '(objective:'+str(self._value)+')'
        else:
            return '(objective:'+str(self._subvars)+')'
            
class nothing:
    def __repr__(self):
        return ''       
